/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package bachelorprojectlog;

import app.bachelorproject.auth.Group;
import app.bachelorproject.auth.User;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import javax.annotation.security.DeclareRoles;
import javax.annotation.security.RolesAllowed;
import javax.ejb.Stateless;
import javax.inject.Inject;
import javax.json.stream.JsonParser;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.ws.rs.Consumes;
import javax.ws.rs.FormParam;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.PUT;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.SecurityContext;
import jdk.nashorn.internal.parser.JSONParser;
import org.eclipse.microprofile.jwt.JsonWebToken;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;


/**
 *
 * @author marti
 */

@Path("log")
@Stateless
@DeclareRoles({Group.USER, Group.ADMIN})
public class LogEntryService {
    
    @Context
    SecurityContext sc;
    
    @PersistenceContext
    EntityManager em;
    
    @Inject
    JsonWebToken principal;
    
    @GET
    @Path("getalllogentries")
    @RolesAllowed({Group.ADMIN})
    public List getAllLogEntries(){
        return em.createNamedQuery("LogEntry.findAllLogEntries", LogEntry.class).getResultList();
    }
    
    @GET
    @Path("getlogentries")
    @RolesAllowed({Group.ADMIN, Group.USER})
    public List getLogEntries(){
        return em.createNamedQuery("LogEntity.findByUserId", LogEntry.class)
                .setParameter("userid", sc.getUserPrincipal().getName())
                .getResultList();
    }
    
    @GET
    @Path("getlogentriestest")
    public String getLogEntriesTest(){
        
        ArrayList<LogEntry> logEntries = new ArrayList<LogEntry>();
        
        return "yolo";
    }
    
    
        
    @POST
    @Path("create")
    @Produces(MediaType.APPLICATION_JSON)
    @RolesAllowed({Group.ADMIN, Group.USER})
    public Response create(
            @FormParam("uid") String userId,
            @FormParam("handelsdag") String buyDate,
            @FormParam("oppgjørsdag") String sellDate,
            @FormParam("verdipapir") String verdipapir,
            @FormParam("transaksjonstype") String type,
            @FormParam("antall") int amount,
            @FormParam("kurs") double kurs,
            @FormParam("beløp") double price,
            @FormParam("resultat") String result
            ){

        Calendar buy = StringToCalendar(buyDate);
        Calendar sell = StringToCalendar(sellDate);
        
        User user = em.find(User.class, principal);

        //LogEntry logEntry = new LogEntry(12, user, buy, sell, type, verdipapir, amount, kurs, price, result, "KEKW");
        
        //em.persist(logEntry);
        
        return Response.ok().build();
    }
    
    @POST
    @Path("createtest")
    @Consumes("application/json")
        public Response tst(String json) throws JSONException {

        JSONArray ja = new JSONArray(json);
        System.out.println(ja);
        ArrayList<LogEntry> logEntries = new ArrayList<LogEntry>();

        for (int i = 0; i < ja.length(); i++){
            //we do this backwards so that the oldest entry gets the lowest id
            JSONObject jo = new JSONObject(ja.getString(ja.length() - i - 1));
            System.out.println(jo);
        }
        return Response.ok(ja.toString()).build();
    }
    
    //takes a string in the form of multiple json objects, converts them and persists them to db
    @POST
    @Path("createwjson")
    @Consumes("application/json")
    @RolesAllowed({Group.ADMIN, Group.USER})
    public Response bar(String json) throws JSONException {

        JSONArray ja = new JSONArray(json);
        System.out.println(ja);
        User user = em.find(User.class, sc.getUserPrincipal().getName());
        ArrayList<LogEntry> logEntries = new ArrayList<LogEntry>();
        for (int i = 0; i < ja.length(); i++){
            //we do this backwards so that the oldest entry gets the lowest id
            try{
            JSONObject jo = new JSONObject(ja.getString(ja.length() - i - 1));
            //System.out.println(jo);
            
            
            
            
            LogEntry logEntry = CreateLogObject(jo, user);
               
            LogEntry le = em.find(LogEntry.class, logEntry.getId());
            if(le == null){
                
                logEntries.add(logEntry);
                em.persist(logEntry);
                em.flush();
                
                List<String> tags = new ArrayList<>();
                Calendar cal = Calendar.getInstance();
                int someInt = logEntry.getHandelsDag().get(Calendar.DAY_OF_WEEK);
                
                String[] strDays = new String[] { "søndag", "mandag", "tirsdag", "onsdag", "torsdag",
                    "fredag", "lørdag" };
                tags.add(strDays[someInt - 1]);
                logEntry.setTags(tags);
                em.merge(logEntry);
                
            }
            }
            catch(Exception e){
                
            }
            
        }
        return Response.ok(logEntries).build();
    }
    
    @PUT
    @Path("addtag")
    @RolesAllowed({Group.ADMIN, Group.USER})
    public Response addTag(@FormParam("id") int id,
            @FormParam("tag") String tag){
        LogEntry le = em.find(LogEntry.class, id);
        if(le.getTags() == null){
            System.out.println("tags is empty");
            List<String> tags = new ArrayList<>();
            tags.add(tag);
            le.setTags(tags);
        }
        else{
            if(!le.getTags().contains(tag)){
                le.getTags().add(tag);
            }
            
            
        }
        em.merge(le);
        return Response.ok(le).build();
    }
    
    @PUT
    @Path("removetag")
    @RolesAllowed({Group.ADMIN, Group.USER})
    public Response removeTag(@FormParam("id") int id,
            @FormParam("tag") String tag){
        LogEntry le = em.find(LogEntry.class, id);
        if((le.getTags() == null) || (le.getTags().isEmpty())){
            System.out.println("tags is empty");
            return Response.status(404, "Tag does not exist").build();
        }
        else{
            le.getTags().remove(tag);
            em.merge(le);
            return Response.ok(le).build();
            
        }
    }
    
    @PUT
    @Path("editcomment")
    @RolesAllowed({Group.ADMIN, Group.USER})
    public  Response editComment(
            @FormParam("id") int id,
            @FormParam("comment") String comment){
        
        LogEntry logEntry = em.find(LogEntry.class, id);
        logEntry.setComment(comment);
        em.merge(logEntry);
        
        return Response.ok(logEntry).build();
    }
    
    
    private LogEntry CreateLogObject(JSONObject jo, User user) throws JSONException{
        System.out.println("trying to make log entry...");
        
            Calendar buy = StringToCalendar("15.05.2020");
            Calendar sell = StringToCalendar("17.05.2020");
            String type = "None";
            String verdiPapir = "None";
            int id = 10;
            int amount = 100;
            double kurs = 10.4;
            double price = 1000000.33;
            String result = "-1";
            String comment = "No Comment.";
            
            try{
                buy = StringToCalendar(jo.getString("Handelsdag"));
                sell = StringToCalendar(jo.getString("Oppgjørsdag"));
            }
            catch(Exception e){
                e.printStackTrace();
            }
            try{
                if(jo.getString("Transaksjonstype") != null && !jo.getString("Transaksjonstype").isEmpty()){
                type = jo.getString("Transaksjonstype");
                }
                if(jo.getString("Verdipapir") != null && !jo.getString("Verdipapir").isEmpty()){
                verdiPapir = jo.getString("Verdipapir");
                }
                
            }
            catch(Exception e){
                e.printStackTrace();
            }
            id = Integer.parseInt(jo.getString("Id"));
            amount = Integer.parseInt(jo.getString("Antall").replace(" ", "").replace(".", ""));
            kurs = Double.parseDouble(jo.getString("Kurs").replace(" ", ""));
            price = Double.parseDouble(jo.getString("Beløb").replace(" ", ""));
            result = jo.getString("Resultat").replace(" ", "");
            System.out.print(sell);
            System.out.print(buy);
            System.out.print(type);
            System.out.print(verdiPapir);
            System.out.print(id);
            System.out.print(amount);
            System.out.print(kurs);
            System.out.print(price);
            System.out.print(result);
            System.out.print(comment);
            System.out.print("-------------------------------------------------");
            
            LogEntry logEntry = new LogEntry(id, user, buy, sell, type, verdiPapir , amount, kurs, price, result, comment);
            System.out.println("OBJEKT == " + logEntry);
            return logEntry;
    }
    
    private Calendar StringToCalendar(String date){
        Calendar returnDate = Calendar.getInstance();
        returnDate.clear();
     
        int day = 0;
        int month = 0;
        int year = 0;
        
        String delims = "[.]";
        String secondDelims ="[-]";
        String[] tokens = date.split(delims);
        if(tokens.length != 3){
            tokens = tokens[0].split(secondDelims);
        }
        
        
        for (int i = 0; i<tokens.length; i++){
            tokens[i].replaceFirst("^0+(?!$)", "");
            if (i == 0){
                day = Integer.parseInt(tokens[i]);
                System.out.println("Calculated day = " + day);
                
            }
            if (i == 1){
                month = Integer.parseInt(tokens[i]);
                
            }
            if (i == 2){
                year = Integer.parseInt(tokens[i]);
                
            }
        }
        
        returnDate.set(year, month - 1, day);
        return returnDate;
    }
}
